/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */
/*
 *  Copyright © 2007 Pierre Habouzit
 *  Copyright © 2000-2002 Vsevolod Volkov <vvv@mutt.org.ua>
 */

#include <lib-sys/evtloop.h>
#include "pop.h"
#include "account.h"

typedef struct pop_data_t {
    int refcnt;
    job_t *w;

    ACCOUNT account;
} pop_data_t;
DO_INIT(pop_data_t, pop_data);
DO_WIPE(pop_data_t, pop_data);
static void pop_data_delete(pop_data_t **tp);
DO_ARRAY_TYPE(pop_data_t, pop_data);
DO_ARRAY_FUNCS(pop_data_t, pop_data, pop_data_delete);

/****************************************************************************/
/* module                                                                   */
/****************************************************************************/

static pop_data_array conns;

static inline pop_data_t *pop_data_new(void) {
    pop_data_t *res = pop_data_init(p_new(pop_data_t, 1));
    res->refcnt = 1;
    pop_data_array_append(&conns, res);
    return res;
}
static inline pop_data_t *pop_data_dup(pop_data_t *t) {
    t->refcnt++;
    return t;
}
static void pop_data_delete(pop_data_t **tp) {
    if (!*tp)
        return;
    if (--(*tp)->refcnt > 0) {
        *tp = NULL;
    } else {
        for (int i = 0; i < conns.len; i++) {
            if (conns.arr[i] == *tp) {
                pop_data_array_take(&conns, i);
                break;
            }
        }
        pop_data_wipe(*tp);
        p_delete(tp);
    }
}

static __init void pop_initialize(void)
{
    pop_data_array_init(&conns);
}
static __fini void pop_shutdown(void)
{
    pop_data_array_wipe(&conns);
}

/****************************************************************************/
/* pop3 machine                                                             */
/****************************************************************************/

static int pop_setup(job_t *w, void *cfg)
{
    w->ptr = cfg;
    /* FIXME */ return el_job_release(w, EL_ERROR);
    return 0;
}

static void pop_finalize(job_t *w, el_status reason)
{
    pop_data_delete((pop_data_t **)(void *)&w->ptr);
}

static machine_t const pop_machine = {
    .name     = "POP3",
    .setup    = &pop_setup,
    .finalize = &pop_finalize,
};

/****************************************************************************/
/* pop3 mx driver                                                           */
/****************************************************************************/

mx_t const pop_mx_ng = {
    .type = M_POP,
};
